@@ -4,7 +4,7 @@ use warnings;
 
 use base 'Test2::Compare::Base';
 
-our $VERSION = '0.000035';
+our $VERSION = '0.000037';
 
 use Test2::Util::HashBase qw/checks _reduction/;
 
@@ -43,7 +43,7 @@ sub set_reduction {
 sub verify {
     my $self = shift;
     my %params = @_;
-    return $params{exists} ? 1 : 0;
+    return 1;
 }
 
 sub add_check {
